/*
 * $Id: Autoupdate.c 366 2020-09-08 13:48:58Z gingko $
 * Copyright (C) 2010 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Application de mise  jour de Pouchin TV Mod
 *
 *	Ce fichier gnre une application minimale, destine  tre excute avec des
 *	droits d'accs plus levs que Pouchin TV Mod, et permettre ainsi de grer
 *	correctement le contrle de compte utilisateur de Windows Vista/7/8/10.
 *
 *	L'application est compile sans le support CRT du langage C, ce qui permet de
 *	gnrer un code excutable minuscule. Le fichier binaire rsultant est ensuite
 *	compress (LZMA) et inclus dans les ressources de Pouchin TV Mod, qui se
 *	chargera ensuite de l'en extraire et dcompresser au moment o il sera
 *	ncessaire.
 **/

#include <tchar.h>
#include <windows.h>

/// Substitut  strchr (les bibliothques standards du C n'tant pas disponibles)
LPCTSTR my_strchr(LPCTSTR pszStr, TCHAR c)
{
	if (pszStr) {
		TCHAR nCh;

		for (; nCh = *pszStr; ++pszStr) {
			if (nCh == c)
				return pszStr;
		}
	}
	return NULL;
}

/// Substitut  strlen (les bibliothques standards du C n'tant pas disponibles)
size_t my_strlen(LPCTSTR pszStr)
{
	LPCTSTR pszPtr = pszStr;

	if (!pszStr)
		return 0;

	while (*pszPtr)
		++pszPtr;
	return pszPtr - pszStr;
}

/**
 * \brief Substitut  strcpy (les bibliothques standards du C n'tant pas disponibles)
 *
 * \param[in]	pszDst	Tampon de rception de la chane destination
 * \param[in]	nCount	Taille du tampon de rception
 * \param[in]	pszSrc	Chane source
 * \return		Pointeur sur la chane destination
 * \note		Si le rsultat excde la taille du tampon de destination, alors la
 *				chane rsultante est tronque.
 **/
LPTSTR my_strcpy(LPTSTR pszDst, size_t nCount, LPCTSTR pszSrc)
{
	if (nCount > 0 && pszDst) {
		--nCount;	// Rserver une position pour le zro terminateur
		if (pszSrc) {
			LPTSTR pszPtr = pszDst;
			size_t nInx;

			for (nInx = 0; nInx < nCount; ++nCount) {
				TCHAR nCh = *pszSrc++;

				if (nCh == 0)
					break;
				*pszPtr++ = nCh;
			}
			*pszPtr = 0;
		}
	}
	return pszDst;
}

/**
 * \brief Substitut  strcat (les bibliothques standards du C n'tant pas disponibles)
 *
 * \param[in]	pszDst	Tampon de rception de la chane destination
 * \param[in]	nCount	Taille du tampon de rception
 * \param[in]	pszSrc	Chane source
 * \return		Pointeur sur la chane destination
 * \note		Si le rsultat excde la taille du tampon de destination, alors la
 *				chane rsultante est tronque.
 **/
LPTSTR my_strcat(LPTSTR pszDst, size_t nCount, LPCTSTR pszSrc)
{
	size_t nLen = my_strlen(pszDst);

	if (nLen < nCount)
		my_strcpy(pszDst + nLen, nCount - nLen, pszSrc);
	return pszDst;
}

/// Sauter les espaces dans une chane de commande
LPCTSTR SkipSpaces(LPCTSTR pszStr)
{
    TCHAR nCh;

    while ((nCh = *pszStr) && nCh <= ' ')
		++pszStr;
	return pszStr;
}

/**
 * \brief Extraction d'un jeton de la ligne de commande
 *
 * Extrait une chane de caractres, ventuellement dlimite par des guillemets.
 * S'il y a des guillemets, ils sont retirs dans la chane destination.
 * S'il n'y a pas de guillemets, alors les caractres ' ' et '=' sont considrs
 * comme des terminateurs.
 * \param[in]	pszDst	Tampon de rception de la chane destination
 * \param[in]	nCount	Taille du tampon de rception
 * \param[in]	pszSrc	Chane source
 * \return		Pointeur sur la suite de la ligne de commande
 * \note		Si le rsultat excde la taille du tampon de destination, alors la
 *				chane rsultante est tronque.
 **/
LPCTSTR GetCmdToken(LPTSTR pszDst, size_t nCount, LPCTSTR pszSrc)
{
	if (nCount > 0 && pszDst) {
		--nCount;	// Rserver une position pour le zro terminateur
		if (*pszSrc) {
			TCHAR	nDelim1 = ' ';
			TCHAR	nDelim2 = '=';
			size_t nInx;

			pszSrc = SkipSpaces(pszSrc);

			if (*pszSrc == '"') {
				nDelim1 = '"';
				nDelim2 = 0;
				++pszSrc;
			}

			for (nInx = 0; nInx < nCount; ++nCount) {
				TCHAR nCh = *pszSrc;

				if (nCh == nDelim1 || nCh == nDelim2 || (nCh >= 0 && nCh < ' '))
					break;
				*pszDst++ = nCh;
				++pszSrc;
			}
			*pszDst = 0;

			if (nDelim1 == '"' && *pszSrc == '"')
				++pszSrc;
		}
	}
	return pszSrc;
}

/**
 * \brief Mise en place d'un fichier de mise  jour.
 *
 * La mise en place se fait, soit en tant que nouveau fichier, soit en tant que
 * remplacement d'un fichier existant.
 * \param[in]	pszFrom	Fichier source (chemin complet)
 * \param[in]	pszTo	Fichier destination (chemin complet)
 * \return		0 si succs, code d'erreur systme autrement.
 **/
DWORD MoveOrReplace(LPCTSTR pszFrom, LPCTSTR pszTo)
{
	DWORD	dwErr = 0;
    TCHAR	szBck[MAX_PATH];
    BOOL    bOk;

    // Dfinir un nom de backup pour le fichier existant
    my_strcpy(szBck, _countof(szBck), pszTo);
    my_strcat(szBck, _countof(szBck), TEXT("~"));

    // Dtruire un ventuel prcdent fichier de backup (ignorer toute erreur ventuelle)
    bOk = DeleteFile(szBck);

    // Renommer le fichier existant vers le nom de backup. Ceci fonctionne mme si le
    // fichier est ouvert (cas d'une mise  jour d'application, par exemple).
    bOk = MoveFile(pszTo, szBck);

    // Mettre en place le nouveau fichier
	if (!MoveFile(pszFrom, pszTo)) {
		TCHAR	szBuf[MAX_PATH];
		TCHAR	szMsg[MAX_PATH];

		dwErr = GetLastError();
		my_strcpy(szMsg, _countof(szMsg), pszTo);
		my_strcat(szMsg, _countof(szMsg), TEXT("\n"));
		FormatMessage(
			FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			dwErr,
			0, // Langue par dfaut (pourrait tre 0x040c = franais)
			szBuf,
			_countof(szBuf),
			NULL);
		//my_uitoa(szFrom, dwErr);
		my_strcat(szMsg, _countof(szMsg), szBuf);

		MessageBox(NULL, szMsg, TEXT("Cannot update file"), MB_ICONERROR);
	}

	return dwErr;
}

/**
 * \brief	Point d'entre du logiciel
 *
 * On n'utilise pas \p WinMain car le support CRT n'est pas disponible.  la
 * place, on utilise \p WinMainCRTStartup qui est le point d'entre normal du
 * support CRT. 
 **/
void WinMainCRTStartup()
{
	LPCTSTR pszCmdLine;
	TCHAR	szFrom[MAX_PATH];
	TCHAR	szTo[MAX_PATH];
	DWORD	dwErr = 0;
	HANDLE	hArgFile;
	DWORD	nRead;
	HLOCAL	hMem = NULL;
	LPTSTR	pszArgs = NULL;

#ifdef _DEBUG
	//DebugBreak();
#endif

	// Obtenir le contenu de la ligne de commande
	pszCmdLine = GetCommandLine();

	// Sauter le nom de l'excutable
	pszCmdLine = GetCmdToken(szFrom, _countof(szFrom), pszCmdLine);
	pszCmdLine = SkipSpaces(pszCmdLine);

	// Obtenir le nom du fichier de liste d'arguments
	pszCmdLine = GetCmdToken(szFrom, _countof(szFrom), pszCmdLine);

	// Lire la liste d'arguments (qui est une liste de la forme :
	// "<fichier source>"="<fichier destination>" + fin de ligne,
	// rpte pour chaque fichier  dplacer.
	hArgFile = CreateFile(szFrom, GENERIC_READ, FILE_SHARE_READ, NULL,
						  OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hArgFile != INVALID_HANDLE_VALUE) {
		DWORD dwSize = GetFileSize(hArgFile, NULL);

		// Allouer la mmoire pour la liste
		hMem = LocalAlloc(0, dwSize+1);
		if (hMem) {
			pszArgs = LocalLock(hMem);
			if (pszArgs) {
				// Charger la liste dans l'espace allou
				if (!ReadFile(hArgFile, pszArgs, dwSize, &nRead, NULL))
					dwErr = GetLastError();
				else
					pszArgs[dwSize/sizeof(TCHAR)] = 0;	// Zro terminateur
			}
		}
	} else
		dwErr = GetLastError();

	if (pszArgs) {
		LPCTSTR pszPtr = pszArgs;

		// Appliquer le dplacement  chaque paire de fichiers dans la liste
		while (dwErr == 0 && *pszPtr) {
			// Lecture du premier nom de fichier
			pszPtr = GetCmdToken(szFrom, _countof(szFrom), pszPtr);
			pszPtr = SkipSpaces(pszPtr);
			if (*pszPtr == '=') {
				++pszPtr;
				// Lecture du second nom de fichier
				pszPtr = GetCmdToken(szTo, _countof(szTo), pszPtr);

				// Dplacer
				dwErr = MoveOrReplace(szFrom, szTo);
			}
			pszPtr = SkipSpaces(pszPtr);
		}
		CloseHandle(hArgFile);
		LocalUnlock(hMem);
	}

	if (hMem)
		LocalFree(hMem);
	ExitProcess(dwErr);
}
