/*
 * $Id: pmtfilter.h 367 2020-09-09 11:22:35Z gingko $
 * Copyright (C) 2006 Pouchin <pouchinteve@free.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Interface du filtre (interne) PMT.
 *
 *	Ce fichier dfinit l'interface du filtre (interne) PMT, qui sert 
 *	dtecter  la vole les changements de caractristiques survenant
 *	dans le flux TS de la chane courante.
 **/

#pragma once

#if USE_PMTFILTER

#include <streams.h>

#define PMT_FILTER_NAME "Filtre PMT PTvM"

typedef void (CALLBACK *PmtMonitorCallback)(const ChainePMT & sExpectedPmt, const ChainePMT & sFoundPmt, DWORD_PTR dwUserData);

DECLARE_INTERFACE_IID_(IPMTMonitor, IUnknown, "42840CF1-DC5B-42e2-B69E-CEDF560109A8")
{
	STDMETHOD(StartPmtMonitoring) (const ChainePMT & sExpectedPmt, PmtMonitorCallback fnCallBack, DWORD_PTR dwUserData) PURE;
	STDMETHOD(StopPmtMonitoring) () PURE;
};

class DECLSPEC_UUID("A72DA4DF-8931-4598-97C5-0AB895EEB356") CPMTFilter : public CBaseRenderer, public IPMTMonitor
{
  #ifdef _DEBUG
	UINT				m_nPmtCount;	//!< Comptage des tables PMT reues
  #endif
	PmtMonitorCallback	m_fnCallBack;
	ChainePMT			m_sExpectedPmt;
	CCritSec			m_cPmtLock;		//!< Section critique de protection des changements
	UINT				m_PmtVersion;
	DWORD_PTR			m_dwUserData;
	UINT				m_nGraceCount;	//!< Dcompte de paquets  ignorer (temps de grce aprs dmarrage)

public:
	CPMTFilter(IUnknown *pUnk, HRESULT *phr);
	virtual ~CPMTFilter();

	// IUnknown
	DECLARE_IUNKNOWN;
	STDMETHODIMP	NonDelegatingQueryInterface(REFIID riid, void ** ppv);

	// Derived classes MUST override these
	HRESULT DoRenderSample(IMediaSample *pMediaSample);
	HRESULT CheckMediaType(const CMediaType *pmt);

	STDMETHODIMP	StartPmtMonitoring(const ChainePMT & sExpectedPmt, PmtMonitorCallback fnCallBack, DWORD_PTR dwUserData);
	STDMETHODIMP	StopPmtMonitoring();

	static LPCWSTR				szFilterName;	// Ne PAS convertir en LPCTSTR (TCHAR)
	static CUnknown * WINAPI	CreateInstance(IUnknown *pUnk, HRESULT *phr);
};

#endif // #if USE_PMTFILTER
