/*
 * $Id: LCD.cpp 251 2010-04-11 09:05:06Z gingko $
 * Copyright (C) 2007 lolo_32 <pouchintv-dev@baysse.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Implmentation de l'affichage sur clavier Logitech
 *
 *	Ce fichier implmente les fonctions ncessaires pour pouvoir utiliser l'affichage
 *	existant sur les crans LCD des claviers de type Logitech G15 et drivs qui
 *	incluent un affichage LCD.
 *
 *	Cet affichage est utilis pour rpliquer les informations dj affiches sur la
 *	fentre, dans l'OSD et dans la zone de notification.
 *	\see http://en.wikipedia.org/wiki/Logitech_G15
 *	\see http://www.logitech.com/index.cfm/keyboards/keyboard/devices/3498
 **/

// Peut tre test par tous
//
// Il faut simplement installer les pilotes du clavier logitech G15 ou des enceintes Z10

#include "stdafx.h"
#include "LCD.h"

CLcd::CLcd() :
	 m_hdc(NULL)
	,m_initialise(false)
	,m_bitmap(NULL)
	,m_connectContext()
	,m_lcdDeviceDesc()
	,m_openContext()
	,m_police(NULL)
	,m_police_heure(NULL)
	,m_mute(false)
	,m_volume(-1)
{
	// Initialise l'cran LCD
	this->m_initialise = InitialiseLcd();

	if (!this->m_initialise) {
		// Dconnecte du LCD car erreur, mais initialisation partielle
		if (this->m_connectContext.connection)
			lgLcdDisconnect(this->m_connectContext.connection);

		// Dsinitialise le LCD (celui-ci ne sera plus utilis)
		lgLcdDeInit();
		return;
	}

	// Dfinit la police par dfaut  utiliser ainsi que ses attributs
	// Taille de la police
	LOGFONT lf;
	ZeroMemory(&lf, sizeof(lf));
	GetObject(m_police, sizeof(lf), &lf);
	lf.lfHeight = 12;
	strcpy_T(lf.lfFaceName, TEXT("Arial"));
	this->m_police = CreateFontIndirect(&lf);
	lf.lfHeight = 11;
	this->m_police_heure = CreateFontIndirect(&lf);

	// Dfinit l'image
	this->m_bitmap = CreateBitmap(this->m_lcdDeviceDesc.Width, this->m_lcdDeviceDesc.Height, 0, this->m_lcdDeviceDesc.Bpp, NULL);

	// Cre un hDc
	this->m_hdc = CreateCompatibleDC(NULL);

	// Associe le BMP et les couleurs
	SelectObject(this->m_hdc, this->m_bitmap);
	SetTextColor(this->m_hdc, RGB(255,255,255));

	SetBkMode(this->m_hdc, TRANSPARENT);
}

CLcd::~CLcd() {
	// Libre les resources associes
	if (!this->m_initialise)
		return;

	lgLcdClose(this->m_openContext.device);
	lgLcdDisconnect(this->m_connectContext.connection);
	lgLcdDeInit();

	DeleteObject(this->m_bitmap);
	this->m_bitmap = NULL;

	DeleteDC(this->m_hdc);
	this->m_hdc = NULL;

	DeleteFont(this->m_police);
	this->m_police = NULL;

	DeleteFont(this->m_police_heure);
	this->m_police_heure = NULL;
}

/**
 * Initialise le Lcd pour utilisation
 *
 * \retval  true   en cas de succs
 * \retval  false  en cas d'erreur
 **/
bool CLcd::InitialiseLcd() {
	DWORD ret = 0;

	ret = lgLcdInit();
	if (ret != ERROR_SUCCESS)
		// Erreur d'initialisation de la bibliothque
		return false;

	// Initialisation de la bibliothque OK !

	ZeroMemory(&this->m_connectContext, sizeof(this->m_connectContext));
	// Nom de l'affichage dans le LCD
	this->m_connectContext.appFriendlyName = szAppName;
	// Est-ce que a dmarre automatiquement?
	this->m_connectContext.isAutostartable = FALSE;
	// L'cran doit-il rester affich une fois l'application finie
	this->m_connectContext.isPersistent = FALSE;
	// Pas de configuration  l'aide des boutons
	this->m_connectContext.onConfigure.configCallback = NULL;
	this->m_connectContext.onConfigure.configContext = NULL;
	// Handle de la connexion au LCD
	this->m_connectContext.connection = LGLCD_INVALID_CONNECTION;

	// On se connecte
	ret = lgLcdConnect(&this->m_connectContext);
	if(ret != ERROR_SUCCESS)
		// Pas de LCD install
		return false;

	// Le LCD est install.

	// Recherche d'un LCD en fonctionnement
	ret = lgLcdEnumerate(this->m_connectContext.connection, 0, &this->m_lcdDeviceDesc);
	if (ret != ERROR_SUCCESS)
		// Pas de LCD en fonctionnement
		return false;

	// LCD trouv
	// Ouverture de celui-ci
	ZeroMemory(&this->m_openContext, sizeof(this->m_openContext));
	// Associe l'ouverture  la connexion
	this->m_openContext.connection = this->m_connectContext.connection;
	// Choisie le LCD  utiliser
	this->m_openContext.index = 0; // premier LCD
	// Handle de l'ouverture
	this->m_openContext.device = LGLCD_INVALID_DEVICE;
	// Pas de configuration  l'aide des boutons
	this->m_openContext.onSoftbuttonsChanged.softbuttonsChangedCallback = NULL;
	this->m_openContext.onSoftbuttonsChanged.softbuttonsChangedContext = NULL;

	// On ouvre l'cran
	ret = lgLcdOpen(&this->m_openContext);
	if (ret != ERROR_SUCCESS)
		return false;

	// Tout est OK
	// Maintenant, le LCD est initialis
	return true;
}

/**
 * Raffiche l'cran
 *
 * \retval  ERROR_SUCCESS  en cas de succs
 * \return  toute autre erreur en cas d'chec
 **/
DWORD CLcd::ActualiseLcd() {
	if (!m_initialise)
		return ERROR_NOT_CAPABLE;

	lgLcdBitmap160x43x1 bitmap;
	lgLcdBitmapHeader entete;
	BYTE winbitmap[860];
	DWORD ret;

	entete.Format = LGLCD_BMP_FORMAT_160x43x1;
	bitmap.hdr = entete;

	/**
	 *Appel des fonctions de dessins ici
	 **/
	this->UpdateTexte();


	// Rcupre le bitmap
	GetBitmapBits(this->m_bitmap, 860, &winbitmap);	// 860 = (lcdWidth * lcdHeigh) / 8

	// On convertis le bitmap
	for (int i = 0, j = 0; i < 860; i++)
	{
		bitmap.pixels[j++] = (winbitmap[i] & 0x80) ? 0xFF : 0x00;
		bitmap.pixels[j++] = (winbitmap[i] & 0x40) ? 0xFF : 0x00;
		bitmap.pixels[j++] = (winbitmap[i] & 0x20) ? 0xFF : 0x00;
		bitmap.pixels[j++] = (winbitmap[i] & 0x10) ? 0xFF : 0x00;
		bitmap.pixels[j++] = (winbitmap[i] & 0x08) ? 0xFF : 0x00;
		bitmap.pixels[j++] = (winbitmap[i] & 0x04) ? 0xFF : 0x00;
		bitmap.pixels[j++] = (winbitmap[i] & 0x02) ? 0xFF : 0x00;
		bitmap.pixels[j++] = (winbitmap[i] & 0x01) ? 0xFF : 0x00;
	}

	// actualise l'affichage
	ret = lgLcdUpdateBitmap(this->m_openContext.device, &(bitmap.hdr), LGLCD_ASYNC_UPDATE(LGLCD_PRIORITY_ALERT));
	ZeroMemory(&winbitmap, sizeof(winbitmap));
	SetBitmapBits(this->m_bitmap, 860, &winbitmap);

	return ERROR_SUCCESS;
}

/**
 * Met  jour les textes : Chane, Volume, programmes
 **/
void CLcd::UpdateTexte() {
	TEXTMETRIC	tm;

	// Police par dfaut
	SelectObject(this->m_hdc, this->m_police);

	// Pour avoir la hauteur de la police
	GetTextMetrics(this->m_hdc, &tm);

	// chane en haut  gauche
	SetTextAlign(this->m_hdc, TA_LEFT);
	TextOutA(this->m_hdc, 10, 1, this->m_chaine, (int)strlen_T(this->m_chaine));

	// Volume
	SetTextAlign(this->m_hdc, TA_RIGHT);
	char volume[46];
	if (this->m_mute)
		sprintf_s(volume, _countof(volume), "Muet (%i)", this->m_volume);
	else
		sprintf_s(volume, _countof(volume), "Volume %i", this->m_volume);
	TextOutA(this->m_hdc, this->m_lcdDeviceDesc.Width-1, 1, volume, (int)strlen_T(volume));

	// Alignement  gauche
	SetTextAlign(this->m_hdc, TA_LEFT);

	// Programme actuel et suivant
	for (int nEmi=0; nEmi <_countof(m_act_next.apEmi); ++nEmi) {
		TCHAR				heure[32]	= TEXT("");	/// heure de dbut de diffusion
		char				prog[256]	= "...";	/// nom du programme
		const Emission *	pEmis		= m_act_next.get(nEmi);
		int					nXStart		= (tm.tmHeight + 1)*(nEmi+1);

		if (pEmis) {
			TimeToStr(pEmis->debut, heure, _countof(heure));
			strcpy_T(prog, pEmis->getName().c_str());
		}

		SelectObject(this->m_hdc, this->m_police_heure);
		TextOut(m_hdc, 1, nXStart+1, heure, strlen_T(heure));
		SelectObject(this->m_hdc, this->m_police);
		TextOutA(this->m_hdc, tm.tmAveCharWidth * 8, nXStart, prog, (int)strlen_T(prog));
	}
}

/**
 * Dfinit le nom de la chane actuelle
 *
 * \param[in]  nomChaine  Nom de la chane
 **/
void CLcd::SetChaine(const char *nomChaine){
	if (!m_initialise)
		return; // Pas initialis, donc, pas besoin de perdre du temps ici

	strncpy_s(m_chaine, _countof(m_chaine), nomChaine, _countof(m_chaine)-1);
}

void CLcd::SetCanal(const Chaine &canal)
{
	if (!m_initialise)
		return; // Pas initialis, donc, pas besoin de perdre du temps ici

	SetChaine(canal.nom);

	m_act_next = canal.getCurrentAndNext();
}
