/*
 * $Id: lzma.cpp 257 2010-08-03 07:35:33Z gingko $
 * Copyright (C) 2010 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Gestion de la dcompression LZMA
 *
 *	Ce fichier contient la fonctionnalit ncessaire  la dcompression de
 *  blocs de mmoire compresss  la norme LZMA (7-Zip).
 **/

#include "stdafx.h"
#include "lzma.h"

// pour dcompresser le LZMA
extern "C" {
	#include "lzma/LzmaDec.h"
	#include "lzma/Types.h"
}

/// Allocateur de mmoire pour dcompresser les fichiers LZMA
static void *SzAlloc(void *p, size_t size) { p = p; if (size == 0) return 0; return malloc(size); }
/// Librateur de mmoire pour dcompresser les fichiers LZMA
static void SzFree(void *p, void *address) { p = p; free(address); }
/// Structure dfinissant l'allocateur/librateur
static ISzAlloc lzmaAlloc = { SzAlloc, SzFree };


/**
 * Dcompresse les donnes LZMA
 *
 * \param[out] vDecompresse	Vecteur d'octets initialiss dans la fonction contenant les donnes dcompresses
 * \param[in]  vCompresse	Vecteur d'octets des donnes compresses
 * \return					Indique le statut de la dcompression
 **/
bool DecompressLzma(ByteVector & vDecompresse, const ByteVector & vCompresse)
{
	SRes		res;								// Rsultat des fonctions
	SizeT		outTraite	= 0;					// Nombre d'octets en sortie traits
	const int	tailleHeader = LZMA_PROPS_SIZE +8;	// Taille de l'en tte ( ignorer pour les donnes)
	SizeT		inTraite	= vCompresse.size();	// Nombre d'octets en lecture traits
	CLzmaDec	state;								// tat de la dcompression
	ELzmaStatus	status;								// Statut de la dcompression

	// On rcupre la taille du fichier dcompress
	UInt32	tailleDecompresse = 0;

	for (int i = 0; i < 8; i++)
		tailleDecompresse += (UInt32)vCompresse.get()[LZMA_PROPS_SIZE + i] << (i * 8);
	outTraite = tailleDecompresse;

	vDecompresse.resize(tailleDecompresse);

	// Initialise le dcodeur
	LzmaDec_Construct(&state);
	res = LzmaDec_Allocate(&state, &vCompresse.get()[0], LZMA_PROPS_SIZE, &lzmaAlloc);
	LzmaDec_Init(&state);
	// On dcode
	res = LzmaDec_DecodeToBuf(&state, &vDecompresse.get()[0], &outTraite, &vCompresse.get()[tailleHeader], &inTraite, LZMA_FINISH_END, &status);
	// On nettoye la mmoire
	LzmaDec_Free(&state, &lzmaAlloc);

	return status == LZMA_STATUS_FINISHED_WITH_MARK && !vDecompresse.empty();
}
