/*
 * $Id: mediadefs.cpp 264 2010-11-01 22:49:12Z gingko $
 * Copyright (C) 2010 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Implmentations lies aux dfinitions unifies de mdias
 *
 *	Ce fichier implmente des fonctions de conversion et de reprsentation
 *	de types de donnes unifis.
 **/

#include "stdafx.h"
#include "mediadefs.h"

/// Conversion des types de mdia MPEG en types de mdia locaux
MediaType MpegType2MediaType(StreamType eType)
{
	switch (eType) {
	case st_MPEG_1_video:
		return mty_Video_Mpeg1;
	case st_MPEG_2_video:
		return mty_Video_Mpeg2;
	case st_MPEG_1_audio:
		return mty_Audio_Mpeg1;
	case st_MPEG_2_audio:
		return mty_Audio_Mpeg2;
	case st_PES_private:
		return mty_Private;
	case st_AAC_audio:
	case st_AAC_audio_2:
	case st_AAC_audio_3:
		return mty_Audio_Aac;	/// \todo Affiner
	case st_H264_video:
		return mty_Video_H264;
	case st_AC3_audio:
		return mty_Audio_Ac3;
	case st_EAC3_audio:
		return mty_Audio_EAc3;
	}
	return mty_Other;
}

const AssocElement aMediaTypeTable[] =
{									// MediaType
	{TEXT("(undef)"),				mty_Undef},
	{TEXT("(unknown)"),				mty_Unknown},
	{TEXT("SFN"),					mty_SFN},
	{TEXT("PAT"),					mty_SI_pat},
	{TEXT("CAT"),					mty_SI_cat},
	{TEXT("NIT"),					mty_SI_nit},
	{TEXT("SDT"),					mty_SI_sdt},
	{TEXT("EIT"),					mty_SI_eit},
	{TEXT("PMT"),					mty_SI_pmt},
	{TEXT("SI (private)"),			mty_SI_private},
	{TEXT("SI (unknown)"),			mty_SI_unknown},
	{TEXT("Video"),					mty_Video},
	{TEXT("Video Mpeg1"),			mty_Video_Mpeg1},
	{TEXT("Video Mpeg2"),			mty_Video_Mpeg2},
	{TEXT("Video Mpeg2 crypt"),	mty_Video_Mpeg2_crypt},
	{TEXT("Video H264"),			mty_Video_H264},
	{TEXT("Video H264 crypt"),		mty_Video_H264_crypt},
	{TEXT("Audio"),					mty_Audio},
	{TEXT("Audio Mpeg1"),			mty_Audio_Mpeg1},
	{TEXT("Audio Mpeg2"),			mty_Audio_Mpeg2},
	{TEXT("Audio Mpeg1 Layer I"),	mty_Audio_Mpeg1_L1},
	{TEXT("Audio Mpeg1 Layer II"),	mty_Audio_Mpeg1_L2},
	{TEXT("Audio Mpeg1 Layer III"),	mty_Audio_Mpeg1_L3},
	{TEXT("Audio Mpeg2 Layer III"),	mty_Audio_Mpeg2_L3},
	{TEXT("Audio AAC"),				mty_Audio_Aac},
	{TEXT("Audio AC3"),				mty_Audio_Ac3},
	{TEXT("Audio AC3 HC"),			mty_Audio_Ac3_hc},
	{TEXT("Audio AC3 atsc"),		mty_Audio_Ac3_atsc},
	{TEXT("Audio E-AC3"),			mty_Audio_EAc3},
	{TEXT("Audio E-AC3 HC"),		mty_Audio_EAc3_hc},
	{TEXT("Audio E-AC3 atsc"),		mty_Audio_EAc3_atsc},
	{TEXT("PES (private)"),			mty_Private},
	{TEXT("Subtitles"),				mty_Subtitling},
	{TEXT("Teletext"),				mty_Teletext},
	{TEXT("PES (unknown)"),			mty_Other},
	{NULL, 0}				// -> terminateur
};

/// Table limite aux seuls flux audio
const AssocElement aAudioTypeTable[] =
{									// MediaType
	{TEXT("Mpeg1"),					mty_Audio_Mpeg1},
	{TEXT("Mpeg2"),					mty_Audio_Mpeg2},
	{TEXT("Mpeg1 Layer I"),			mty_Audio_Mpeg1_L1},
	{TEXT("Mpeg1 Layer II"),		mty_Audio_Mpeg1_L2},
	{TEXT("Mpeg1 Layer III"),		mty_Audio_Mpeg1_L3},
	{TEXT("Mpeg2 Layer III"),		mty_Audio_Mpeg2_L3},
	{TEXT("AAC"),					mty_Audio_Aac},
	{TEXT("AC3"),					mty_Audio_Ac3},
	{TEXT("AC3 HC"),				mty_Audio_Ac3_hc},
	{TEXT("AC3 atsc"),				mty_Audio_Ac3_atsc},
	{TEXT("E-AC3"),					mty_Audio_EAc3},
	{TEXT("E-AC3 HC"),				mty_Audio_EAc3_hc},
	{TEXT("E-AC3 atsc"),			mty_Audio_EAc3_atsc},
	{NULL, 0}				// -> terminateur
};

