/*
 * $Id: main.h 395 2020-10-07 16:16:20Z gingko $
 * Copyright (C) 2006 Pouchin <pouchinteve@free.fr>
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Dfinitions lies  l'interface principale de l'application
 *
 *	Ce fichier contient les dfinitions de l'interface principale de
 *	l'application qui sont ncessaires aux autres modules.
 **/

#pragma once

/// Nom de l'application (macro)
#define APPLICATION_NAME	TEXT("Pouchin TV Mod")	// Utiliser 'szAppName' de prfrence

extern TCHAR		szAppName[];		//!< Nom de l'application (pour le titre de la fentre)
extern TCHAR		szAppVersion[];		//!< Numro de version de l'application
extern TCHAR		szAppFileName[];	//!< Nom du binaire de PTVM

extern HINSTANCE	hAppInstance;		//!< Handle d'instance de l'application
extern HWND			hMainWnd;			//!< Handle de la fentre principale de l'application
extern HWND			hMainStatus;		//!< Handle de la barre d'tat de l'application
extern HMENU		hMainMenu;			//!< Handle de la barre de menus de l'application
extern HACCEL		hMainAccel;			//!< Acclrateur gnr en runtime

extern HICON		hCurIcon;			//!< Icne courante (normale, ou bien rouge pour enregistrement)

enum WinVersion
{
	wv_Unknown,
	wv_Older,
	wv_2000,
	wv_XP,
	wv_Server2003,
	wv_Vista,
	wv_Server2008,
	wv_Seven,
	wv_Server2008R2,
	wv_Eight,
	wv_Server2012,
	wv_Eight_dot_1,
	wv_Server2012R2,
	wv_Ten,
	wv_Server2016,
	wv_Newer
};

extern WinVersion	eWinVers;	//!< Version de Windows (dcodage basique)

enum ExitMode
{
	em_Quit,					//!< Sortie normale de l'application (= 0)
	em_FullRestart,				//!< Sortie complte de l'application avec redmarrage
	em_Shutdown,				//!< Sortie de l'application et arrt de l'ordinateur
	em_ShutdownForced			//!< Sortie de l'application et arrt forc de l'ordinateur
};

/// Valeur combinable avec \p ExitMode via \p MAKELPARAM indiquant si on doit ou non
/// vrifier la prsence d'un enregistrement en cours avant de quitter
#define REC_NOCONFIRM 1

extern ExitMode	eExitMode;		//!< Indique si (et comment) l'application doit tre relance

/// Requtes pouvant tre changs entre instances de l'application avec \p nWM_APP_INSTANCES
enum InterInstancesRequests : BYTE		// Valeurs  mettre dans wParam
{
	iir_Null,							//!< 0 = aucun message
	iit_RequestPowerStateChange = 0x10,	//!< Demande si on peut teindre ou mettre en veille l'ordinateur
	iit_DenyPowerStateChange			//!< Rponse d'une instance qui refuse d'teindre ou mettre en veille
};

/**
 * Identifiant du message utilis pour changer entre des requtes entre instances de la
 * mme application. Les requtes elles-mmes sont places dans \p wParam, tandis que
 * \p lParam contiendra la handle de la fentre principale de l'instance qui envoie.
 **/
extern UINT			nWM_APP_INSTANCES;

/// Mis  \p true si une autre instance a refus l'arrt ou la mise en veille
extern bool			bPowerStateChangeDenied;

/// Diverses informations sur la version de Windows
extern OSVERSIONINFOEX windows_version;

/// Blocage de l'extinction du fait des enregistrements
class CShutdownBlockReason
{
	enum SdMode
	{
		sdm_NoBlock,
		sdm_WaitRecording,
		sdm_Recording
	};

	typedef BOOL (WINAPI * PFN_SHUTDOWNBLOCKREASONCREATE) (HWND, LPCWSTR);
	typedef BOOL (WINAPI * PFN_SHUTDOWNBLOCKREASONDESTROY) (HWND);

	SdMode							eSdm;
	PFN_SHUTDOWNBLOCKREASONCREATE	fnShutdownBlockReasonCreate;
	PFN_SHUTDOWNBLOCKREASONDESTROY	fnShutdownBlockReasonDestroy;
public:
	CShutdownBlockReason();

	void	Update();
	void	Reset();
	bool	isAvailable() const {
				return fnShutdownBlockReasonCreate != NULL && fnShutdownBlockReasonDestroy != NULL; }

	~CShutdownBlockReason() {
				Reset(); }
};

/// Blocage de l'extinction du fait des enregistrements
extern CShutdownBlockReason cShutBlockReason;

/** @defgroup TIMER_codes Liste des codes de timers dfinis
 *  @{
 **/

enum TIMER_codes
{
	TIMER_VIDEO_MOVE = 120,	//!< Limitation de la frquence des mises  jour de la position de la vido
	TIMER_QUIT,				//!< Utilis quand un enregistrement provoque l'arrt de l'application
	TIMER_SHUTDOWN,			/*!< Utilis quand un enregistrement provoque l'extinction de l'ordinateur
								 (HIWORD != 0 pour extinction force) */
	TIMER_SLEEP,			/*!< Utilis quand un enregistrement provoque la mise en veille de l'ordinateur
								 (HIWORD != 0 pour veille prolonge) */
	TIMER_NUMBER,			//!< Temps d'attente entre chiffres lors de la saisie de numro de chane
	TIMER_BORDER,			//!< Discrimination entre clic pour plein cran et clic pour bordures
	TIMER_ENABLE_OSD,		//!< Timer de ractivation de l'OSD si aucun autre vnement ne l'a fait avant
	TIMER_UPDATE_OSD,		//!< Temps avant recalcul OSD
	TIMER_OSD,				//!< Temps d'expiration de l'OSD
	TIMER_CURSOR,			//!< Temps de maintien du curseur aprs mouvement
	TIMER_DELAYED_RECORD,	//!< Gestion du dmarrage des enregistrements programms
	TIMER_DELAYED_STOP,		//!< Gestion de l'arrt programm
	TIMER_EVR_UPDATE,		//!< Utilis pour la mise  jour de l'affichage EVR
	TIMER_ZAPPING,			/*!< Dlai entre changement chane et affichage infos
								 (HIWORD != 0 pour 2me essai) */
	TIMER_SIGNAL,			//!< Priodicit de le mise  jour des informations de qualit du signal
	TIMER_SIGNAL_ADJ,		//!< Timer d'ajustement de frquence
	TIMER_EPG_UPDATE,		//!< Limitation de la frquence des mises  jour de l'EPG
	TIMER_CHAN_LVDRAG,		//!< Cadencement du scrolling de la rorganisation des chanes
	TIMER_NOTIFY_UPDATE,	//!< Auto-mise  jour du tooltip de l'icne de notification et du message MSN
	TIMER_EPG_EVENT,		//!< Temps avant prochaine expiration horaire dans l'EPG
	TIMER_WAKEUP			//!< Temps entre la sortie du mode veille et la relance du graphe
};

/// Timer de scurit pour la ractivation de l'OSD : normalement on s'attend  ce que
/// certains vnements du graphe provoquent cette ractivation, mais il arrive qu'aucun
/// vnement de ce type ne se produise, auquel cas la ractivation se fera au bout de ce temps.
#define			DUREE_ENABLE_OSD	4000

/// Nombre d'essais de redmarrage du graphe avec TIMER_WAKEUP
#define WAKEUP_RETRIES 16

/// Dcompte d'essais de redmarrage du graphe
static BYTE	nWakeupRetryCount = WAKEUP_RETRIES;

/** @} */ // fin de TIMER_codes

/// Liste des messages locaux  l'application
enum WM_APP_codes
{
	/**
	 * Notification d'vnements transmis depuis l'icne de la barre des tches
	 * \param[in]	wParam			Voir documentation Microsoft de \p NOTIFYICONDATA::uCallbackMessage
	 * \param[in]	lParam			Voir documentation Microsoft de \p NOTIFYICONDATA::uCallbackMessage
	 **/
	WM_APP_TRAYICON_MSG = WM_APP,

	/**
	 * Notification de changement dans le guide de programmation (EPG), ncessitant
	 * que la liste soit reconstruite.
	 **/
	WM_APP_EPG_UPDATED,

	/**
	 * Notification de changement dans le format du flux TS de la chane, dtect par le filtre PMT
	 **/
	WM_APP_PMT_UPDATED,

	/**
	 * Reconstruction de la liste des enregistrements programms aprs changement
	 **/
	WM_APP_PROG_CHANGED,

	/**
	 * Notification d'une vnement dans le guide de programmation (EPG),
	 * ncessitant seulement une mise  jour des couleur
	 **/
	WM_APP_EPG_EVENT,

	/**
	 * Traitement des messages d'information envoys au cours de la recherche
	 * \param[in]	LOBYTE(wParam)	Code de notification
	 * \param[in]	wParam			Autres parties : voir #ScanNotifications
	 * \param[in]	lParam			Voir #ScanNotifications
	 **/
	WM_APP_SCAN_NOTIFY,

	/**
	 * Dfinition d'un raccourci clavier par saisie de la commande
	 * \param[in]	wParam			Pointeur sur le raccourci clavier dfini
	 *								(allou en mmoire,  supprimer par \p delete aprs usage)
	 **/
	WM_APP_HOOK_OK,

	/**
	 * Annulation de saisie de raccourci clavier
	 **/
	WM_APP_HOOK_CANCEL,

	/**
	 * Notification d'vnement survenant dans le graphe
	 * \param[in]	lParam	Pointeur (\p IPTvM_GraphBase *) sur le graphe concern
	 **/
	WM_APP_GRAPHNOTIFY,

	/**
	 * Notification de validit du rendu vido
	 **/
	WM_APP_VIDEO_UPDATED,

	/**
	 * Ajout d'un enregistrement programm depuis l'EPG
	 * \param[in]	wParam	Identifiant du descripteur d'mission
	 * \param[in]	lParam	Cl de la chane
	 **/
	WM_APP_ADD_EPG_PROG,

	/**
	 * Syntonisation d'une nouvelle frquence
	 * \param[in]	wParam	Numro de canal
	 * \param[in]	lParam	Nouvelle frquence
	 **/
	WM_APP_NEW_FREQ,

	/**
	 * Notification de dbut ou d'arrt d'enregistrement
	 **/
	WM_APP_RECORD_EVENT,

	/**
	 * Signale qu'une mise  jour a t trouve et peut tre tlcharge
	 * \param[in]	wParam	Raison du blocage
	 **/
	WM_APP_UPDATEFOUND,

	/**
	 * Notification du fait qu'un arrt systme a t bloqu
	 * \param[in]	wParam	Raison du blocage
	 **/
	WM_APP_SHUTBLOCK_NOTIFY
};

/**
 * Gnration d'une chane de caractres reprsentant la version actuelle de Windows
 * \param[in]	sWv	Structure d'informations pralablement charge
 **/
tstring GetWindowsVersionStr(const OSVERSIONINFOEX & sWv);

/**
 * Obtenir la priorit d'excution courante de l'application
 * \return	prio	Priorit actuelle. Voir fonction systme \p GetPriorityClass.
 **/
DWORD get_priority();

/**
 * Changement de chane, slection par index dans la table.
 * Met galement  jour les menus et la barre d'tat.
 * \param[in]	ixChaine	Index dans la table #Canaux.
 *							(mettre -1 pour dsactiver la slection de chane)
 * \return		Code d'erreur DirectShow
 **/
HRESULT change_chaine_update(int ixChaine);

/**
 * Changement de chane, slection par index dans la table.
 * Met galement  jour les menus, la barre d'tat et l'affichage OSD.
 * \param[in]	ixChaine	Index dans la table #Canaux.
 *							(mettre -1 pour dsactiver la slection de chane)
 * \return		Code d'erreur DirectShow
 **/
HRESULT zappe_index(int ixChaine);

/**
 * Restauration d'une chane sauvegarde par cl.
 * Si la cl n'est pas valide, on utilise la premire de la liste.
 * Retourne 'true' si succs, 'false' si la liste est vide.
 **/
bool restaure_chaine(UINT cleChaine);

// Rcupration de l'tat de la fentre :
EtatsFenetre etat_fenetre(HWND hWnd);

/**
 * Met  jour les raccourcis dans les menus
 **/
void update_shortcuts_menus();

/// Demande de mise  jour de l'OSD
void request_osd_update();

/// Met  jour la commande de menu "Quitter" selon le temps restant avant enregistrement
void update_record_protect(LONG nTimeBeforeNextRecording);
