/*
 * $Id: msn_notify.cpp 251 2010-04-11 09:05:06Z gingko $
 * Copyright (C) 2009 gingko - http://gingko.homeip.net/
 *
 * This file is part of Pouchin TV Mod, a free DVB-T viewer.
 * See http://www.pouchintv.fr/ for updates.
 *
 * Pouchin TV Mod is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Pouchin TV Mod is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 *
 * Most files of this project have been changed from the Pouchin TV
 * project (Copyright (C) 2006 Pouchin <pouchinteve@free.fr>),
 * to use with a modified version of this software.
 * See http://pouchinteve.free.fr/ for the original project.
 */

/** \file
 *	\brief Implmentation de la fonctionnalit de communication avec MSN
 *
 *	Ce fichier implmente les fonctionnalits permettant d'envoyer des messages 
 *	l'application MSN Messenger / Windows Live Messenger.
 **/

#include "stdafx.h"
#include "msn_notify.h"

/**
 * Envoi d'un message dans la fentre MSN
 * \param[in]	hMsnWnd	Handle de la fentre MSN
 * \param[in]	pszMsg	Message  envoyer
 **/
void CMSNMsg::send(HWND hMsnWnd, LPCWSTR pszMsg)
{
	// viter de renvoyer deux fois de suite le mme message (MSN ayant tendance
	//  mettre une notification  chaque fois)
	if (pszMsg!=strLastMsg) {
		COPYDATASTRUCT msndata = {
			0x547,								// dwData
			(strlen_T(pszMsg)+1)*sizeof(WCHAR),	// cbData
			PVOID(pszMsg)						// lpData
		};

		strLastMsg = pszMsg;
		SendMessage(hMsnWnd, WM_COPYDATA, (WPARAM)hMainWnd, (LPARAM)&msndata);
	}
}

/**
 * Envoi du nom de la chane et de l'mission dans la fentre MSN,
 * si l'option correspondante est active
 *
 * Infos : http://www.autoitscript.com/forum/index.php?showtopic=43112
 **/
void CMSNMsg::set()
{
	#define MSN_MAX_LENGTH 256

	if (use_msn && ixChaine_ok(ixChaineCourante)) {
		HWND hMsnWnd = FindWindow(TEXT("MsnMsgrUIManager"), NULL);

		if (hMsnWnd != NULL) {
			WCHAR			buffer[MSN_MAX_LENGTH];
			const Chaine &	canal = Canaux[ixChaineCourante];

			swprintf_s(buffer, _countof(buffer),
				L"\\0Music\\01\\0{0} - {1}\\0%S\\0%" T2w L"\\0\\0\\0",
				canal.nom, canal.getCurrentProgram().getName(TEXT("...")).c_str());

			send(hMsnWnd, buffer);
			msg_set = true;
		}
	}
}

/**
 * Effacement du message pralablement envoy dans la fentre MSN,
 * s'il y en avait un de dfini
 **/
void CMSNMsg::reset()
{
	if (msg_set) {
		// On cherche la fentre
		HWND hMsnWnd = FindWindow(TEXT("MsnMsgrUIManager"), NULL);

		// Si, hMsnWnd == NULL, MSN ne fonctionne pas ou la fentre est masque (donc, pas de mise  jour)
		if (hMsnWnd != NULL) {
			// Si on l'a trouve, on efface la "musique" indique sur MSN
			send(hMsnWnd, L"\\0Music\\00\\0\\0\\0\\0\\0\\0");
		}
		msg_set = false;
	}
}
